#include "base_op.h"

namespace fg {

std::map<std::string, CreatePluginFunc>& PluginRegistry::getRegistry() {
  static std::map<std::string, CreatePluginFunc> registry;
  return registry;
}

void PluginRegistry::registerPlugin(const std::string& name,
                                    CreatePluginFunc func) {
  getRegistry()[name] = func;
}

CreatePluginFunc PluginRegistry::getCreateFunc(const std::string& name) {
  auto& registry = getRegistry();
  auto it = registry.find(name);
  if (it != registry.end()) {
    return it->second;
  }
  return nullptr;
}

}  // end of namespace fg

extern "C" {

const char** get_register_names(size_t* count) {
  static const char** strings = nullptr;

  auto& registry = fg::PluginRegistry::getRegistry();
  *count = registry.size();

  if (strings == nullptr) {
    strings = new const char*[*count];
    int i = 0;
    for (const auto& pair : registry) {
      strings[i++] = pair.first.c_str();
    }
  }
  return strings;
}

void free_register_names(const char** names) {
  if (names) {
    delete[] names;
    names = nullptr;
  }
}

fg::CreatePluginFunc get_plugin_creator(const char* name) {
  return fg::PluginRegistry::getCreateFunc(name);
}

void destroy_plugin(fg::IFeatureOP* plugin) { delete plugin; }
}