#pragma once

#include <glog/logging.h>

#include <sstream>
#define FLUSH_LOG(severity) google::FlushLogFiles(google::severity)

// 辅助函数模板
template <typename T>
void log_helper(std::ostringstream& os, T&& arg) {
  os << std::forward<T>(arg);
}

template <typename T, typename... Args>
void log_helper(std::ostringstream& os, T&& arg, Args&&... args) {
  os << std::forward<T>(arg);
  log_helper(os, std::forward<Args>(args)...);
}

#define ERROR_EXIT(FeatureName, ...)                                  \
  do {                                                                \
    std::ostringstream oss;                                           \
    log_helper(oss, __VA_ARGS__);                                     \
    auto msg = oss.str();                                             \
    LOG(ERROR) << "feature <" << FeatureName << "> " << msg;          \
    google::FlushLogFiles(google::ERROR);                             \
    throw std::runtime_error("feature <" + FeatureName + "> " + msg); \
  } while (false);
