#pragma once
#include <regex>

#include "api/base_op.h"

namespace fg {

using std::string;
using std::vector;

/**
 * @brief 基于正则表达式的字符串替换操作
 */
class RegexReplace : public IFeatureOP {
 public:
  int Initialize(const string& feature_config) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithStrOutputs(const vector<Field>& inputs,
                            vector<string>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithInt32Outputs(const vector<Field>& inputs,
                              vector<int32>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithInt64Outputs(const vector<Field>& inputs,
                              vector<int64>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithFloatOutputs(const vector<Field>& inputs,
                              vector<float>& outputs) override;

  /// @return 状态码，如果为0表示执行成功
  int ProcessWithDoubleOutputs(const vector<Field>& inputs,
                               vector<double>& outputs) override;

 private:
  string feature_name_;
  std::regex patten_;
  string replacement_;
  string default_value_;
};

}  // end of namespace fg

REGISTER_PLUGIN("RegexReplace", RegexReplace);